<?php


namespace Drupal\stackpath;


use Drupal\Core\Http\ClientFactory;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Site\Settings;
use Drupal\Core\Url;
use Psr\Http\Message\ResponseInterface;

/**
 * Class StackPathApi
 *
 * @package Drupal\stackpath
 */
class StackPathApi {

  /**
   * StackPath API URI.
   */
  const STACKPATH_API_URI = "https://gateway.stackpath.com";

  /**
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * @var string
   */
  protected $token;

  /**
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * StackPathApi constructor.
   *
   * @param \Drupal\Core\Http\ClientFactory $clientFactory
   * @param \Drupal\Core\Logger\LoggerChannelInterface $loggerChannel
   */
  public function __construct(ClientFactory $clientFactory, LoggerChannelInterface $loggerChannel) {
    $this->httpClient = $clientFactory->fromOptions(
      [
        'base_uri' => self::STACKPATH_API_URI,
        'timeout' => 30,
      ]
    );
    $this->logger = $loggerChannel;
  }

  /**
   * @param $cache_tag
   *
   * @return bool|string
   */
  public function createHashTag($cache_tag) {
    return substr(md5(Settings::getHashSalt() . $cache_tag), 0, 5);
  }

  /**
   * Adds default parameters.
   *
   * @param $input
   *
   * @return array
   */
  private function prepareRequestParameters($input = []) {
    $parameters = [];
    if (!empty($input)) {
      $parameters['json'] = $input;
    }
    // Default values can be overridden by defining them in the $opts passed to request.
    $default_options = [
      // Unless overridden, every payload will send the Accept header set to application/json.
      'headers' => [
        'Accept' => 'application/json',
      ],
    ];
    // If the bearer token has been retrieved, supply it as the auth header.
    if (isset($this->token)) {
      $default_options['headers']['Authorization'] = sprintf("Bearer %s", $this->token);
    }
    // Recursive merge of custom options over defaults.
    $parameters = array_merge_recursive($default_options, $parameters);
    return $parameters;
  }

  /**
   * @param $client_id
   * @param $client_secret
   *
   * @return bool|mixed|string
   */
  public function getToken($client_id, $client_secret) {
    if (!empty($this->token)) {
      return $this->token;
    }
    $params = [
      'client_id' => $client_id,
      'client_secret' => $client_secret,
      'grant_type' => 'client_credentials',
    ];
    try {
      $response = $this->httpClient->post('/identity/v1/oauth2/token', $this->prepareRequestParameters($params));
      $data = $this->processResponse($response);
      if (!empty($data['access_token'])) {
        $this->token = $data['access_token'];
        return $this->token;
      }
    } catch (\Exception $e) {
      $this->handleException($e);
    }
    return FALSE;
  }

  /**
   * @param $response
   *
   * @return mixed
   */
  private function processResponse(ResponseInterface $response) {
    return Json::decode($response->getBody()->getContents());
  }

  /**
   * @param \Exception $e
   */
  private function handleException(\Exception $e) {
    $this->logger->error($e->getMessage());
  }

  /**
   * Invalidates CDN caches.
   *
   * @param $stack_id
   * @param null $type
   * @param array $invalidations
   */
  public function invalidate($stack_id, $type = NULL, array $invalidations = []) {
    $root_url = new Url('<front>', [], ['absolute' => TRUE]);
    $url_parts = parse_url($root_url->toString());
    $url = $url_parts['scheme'] . '://' . $url_parts['host'];
    $params = ['items' => []];
    switch ($type) {
      case 'tags':
        foreach ($invalidations as $invalidation) {
          $tag = $this->createHashTag($invalidation);
          $purgeSelector = [
            'selectorType' => 'TAG',
            'selectorName' => 'X-Tag',
            'selectorValue' => $tag,
            'selectorValueDelimiter' => ',',
          ];
          $params['items'][] = [
            'url' => $url,
            'purgeSelector' => [$purgeSelector],
            'recursive' => TRUE,
            'invalidateOnly' => FALSE,
          ];
        }
        break;
      case 'url':
        foreach ($invalidations as $invalidation) {
          $params['items'][] = [
            'url' => $invalidation,
            'recursive' => FALSE,
          ];
        }
        break;
      default:
        $params['items'][] = [
          'url' => $url,
          'recursive' => TRUE,
        ];
        break;
    }
    try {
      $params = $this->prepareRequestParameters($params);
      $response = $this->httpClient->post('/cdn/v1/stacks/' . $stack_id . '/purge', $params);
      $data = $this->processResponse($response);
      return $data;
    } catch (\Exception $e) {
      $this->handleException($e);
    }
  }

}
